/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on May 23, 2005
 */
package com.sunlabs.archsim.runner;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import com.sunlabs.archsim.engine.Director;

/**
 * @author ib27688
 *
 * Load a list of operations from an XML file
 */
public class OperationLoader extends DefaultHandler {

	public static final String OPERATION_LIST= "operation_list";

	private Director director= Director.getInstance();
	
	/** Locator provided by the parser */
	private Locator locator;
	
	/** the list of loaded operations */
	private List<Operation> operations;
	
	public OperationLoader() {
		
	} // ClassRegistryLoader constructor
	
	/** load components form a file */
	public List load(String fileName) {
			operations= new LinkedList<Operation>(); 
			SAXParserFactory factory = SAXParserFactory.newInstance();
	        factory.setValidating(true);
	        //factory.setNamespaceAware(true);
	        try {
	            // Parse the input
	            SAXParser saxParser = factory.newSAXParser();
	            saxParser.parse( new File(fileName), this);

	        } catch (SAXParseException spe) {
	           // Error generated by the parser
	           director.error("\n** Parsing error"
	           		+ ", line " + spe.getLineNumber()
					+ ", uri " + spe.getSystemId()
					+ "   " + spe.getMessage() );

	           // Use the contained exception, if any
	           Exception  x = spe;
	           if (spe.getException() != null)
	               x = spe.getException();
	           x.printStackTrace();
	           System.exit(1);

	        } catch (SAXException sxe) {
	            // Error generated by this application
	            // (or a parser-initialization error)
	      
	        		// first take a look at a message associated with
	        		// 
	        		String message= sxe.getMessage();
	        		if (!message.equals("")) {
	        			director.error(message);
	        		}
	        		
	        		Exception  x = sxe;
	        		if (sxe.getException() != null)
	            		x = sxe.getException();
	        		x.printStackTrace();
	        		System.exit(1);

	        } catch (ParserConfigurationException pce) {
	            	// Parser with specified options can't be built
	            	pce.printStackTrace();

	        } catch (IOException ioe) {
	        		// I/O error
	        		ioe.printStackTrace();
	        }		
	        
	        return operations;
	        
	} // loadFile
	
	/** 
	 * The callback that provides a Locator - used to get line numbers
	 * for error messages, for example.
	 */
    public void setDocumentLocator(Locator l) {
    		locator = l;
    }

	
	/**
	 *  prep work to be done at the beginning of the document 
	 */
	public void startDocument() throws SAXException {
		// do nothing
	} // startDocument
	
	/** 
	 * Finishing work to be done at the end of the XML file
	 */
	public void endDocument() throws SAXException {
		// What do we do?
		// nothing at the moment
	} // endDocument
	
	public void startElement (
			String namespaceURI,
			String lName, // local name
			String qName, // qualified name
			Attributes attrs) 
	throws SAXException {
		String eName = lName; // element name
	     if ("".equals(eName)) eName = qName; // namespaceAware = false
	     // if not the root element , record an operation
	     if (!eName.equals(OPERATION_LIST)) {
	     	Operation op= new Operation();
	     	// record the name of the operation
	     	op.name= eName;
     	// get the names and values for the attributes
	     	op.attrNames= new String[attrs.getLength()];
	     	op.attrValues= new String[attrs.getLength()];
	     	for (int i= 0; i < attrs.getLength(); i++) {
	     		String aName= attrs.getLocalName(i);
	     		if (aName.equals("")) aName= attrs.getQName(i);

	     		op.attrNames[i]= aName;
	     		op.attrValues[i]= attrs.getValue(i);
	     	} // for
	     	operations.add(op);
	     } // if not root element
	    
	} // startElement
	
    public void endElement(String namespaceURI,
            String lName, // simple name
            String qName  // qualified name
           )
    throws SAXException {
    		// do nothing
    } // endElement
	
	
	public void characters(char buf[], int offset, int len)
	throws SAXException {
		// do nothing
	} // characters
	
    // treat validation errors as fatal
    public void error(SAXParseException e)
    throws SAXParseException
    {
        throw e;
    }

    // dump warnings too
    public void warning(SAXParseException err)
    throws SAXParseException
    {
        director.error("** Warning"
        		+ ", line " + err.getLineNumber()
            	+ ", uri " + err.getSystemId()
        		+ "   " + err.getMessage());
    } // Warning
    
    /** 
     * Print a warining.
     * @param msg warning message to be displayed.
     */
    protected void printWarning(String msg) {
        director.error("** Warning"
                + ", line " + locator.getLineNumber()
                + ", uri " + locator.getSystemId());
            director.error("   " + msg);   	
    } // msg
	
	public static void main(String[] argv) {
		
		OperationLoader ldr= new OperationLoader();
		
		
		List ops= ldr.load("TestLoadOperations.xml");
		
		Iterator oi= ops.iterator();
		while (oi.hasNext()) {
			Operation op= (Operation)(oi.next());
			ldr.director.info("Operation: " + op.name);
			for (int i= 0; i< op.attrNames.length; i++) {
				ldr.director.info("; attribute name= " + op.attrNames[i]);
				ldr.director.info("; attribute value= " + op.attrValues[i]);				
			}
			ldr.director.info("");
		}
		ldr.director.info("");
		
	} // main
	
} // OperationLoader